"""
@author: Mario Lassnig
@contact: mlassnig@cern.ch
@since: 0.3
@version: $Id: ListSubscriptions.py,v 1.6 2009/10/06 13:23:17 angelos Exp $
"""

import cliutil

from dq2.common import optparse

from dq2.common.cli.DQDashboardTool import DQDashboardTool

from dq2.clientapi.cli.cliutil import getDQ2, option_value

from dq2.subscription.DQSubscriptionConstants import SubscriptionArchivedState


class ListSubscriptions (DQDashboardTool):
    """
    @since: 0.3.0
    """

    usage = "%prog [options] <DATASET>"
    
    version = "$Revision: 1.6 $"
    
    description = "List Subscriptions"

    toolOptions = [
        optparse.make_option(
            "-a", "--archived",
            action="store_true",
            dest="archived",
            default=False,
            help="only list archived subscriptions"
        ),
        optparse.make_option(
            "-u", "--unarchived",
            action="store_true",
            dest="unarchived",
            default=False,
            help="only list unarchived subscriptions"
        ),
        cliutil.opt_dataset_version
    ]
    
    
    def __init__(self):
        """
        @since: 0.3.0
        """
        DQDashboardTool.__init__(self)
        
        
    def _get_parameters (self):
        """
        @since: 0.3.0
        """
        return self.args[0]
        
        
    def execute(self):
        """
        @since: 0.3.0
        """

        dq = getDQ2(self)
        
        dsn = self._get_parameters()
        
        archived = SubscriptionArchivedState.__ANY__
        if self.options.archived:
            archived = SubscriptionArchivedState.ARCHIVE
        if self.options.unarchived:
            archived = SubscriptionArchivedState.UNARCHIVE
        
        ret = dq.listSubscriptions(dsn, version=self.options.version, archived=archived)
        for site in ret:
            print site
        